/*
 * JBoss, Home of Professional Open Source
 * Copyright 2011, Red Hat, Inc. and individual contributors
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.mobicents.protocols.ss7.m3ua.impl.message.ssnm;

import java.nio.ByteBuffer;

import org.mobicents.protocols.ss7.m3ua.impl.message.M3UAMessageImpl;
import org.mobicents.protocols.ss7.m3ua.impl.parameter.ParameterImpl;
import org.mobicents.protocols.ss7.m3ua.message.MessageClass;
import org.mobicents.protocols.ss7.m3ua.message.MessageType;
import org.mobicents.protocols.ss7.m3ua.message.ssnm.SignallingCongestion;
import org.mobicents.protocols.ss7.m3ua.parameter.AffectedPointCode;
import org.mobicents.protocols.ss7.m3ua.parameter.ConcernedDPC;
import org.mobicents.protocols.ss7.m3ua.parameter.CongestedIndication;
import org.mobicents.protocols.ss7.m3ua.parameter.InfoString;
import org.mobicents.protocols.ss7.m3ua.parameter.NetworkAppearance;
import org.mobicents.protocols.ss7.m3ua.parameter.Parameter;
import org.mobicents.protocols.ss7.m3ua.parameter.RoutingContext;

/**
 * 
 * @author amit bhayani
 * 
 */
public class SignallingCongestionImpl extends M3UAMessageImpl implements SignallingCongestion {

	public SignallingCongestionImpl() {
		super(MessageClass.SIGNALING_NETWORK_MANAGEMENT, MessageType.SIGNALING_CONGESTION,
				MessageType.S_SIGNALING_CONGESTION);
	}

	public AffectedPointCode getAffectedPointCodes() {
		return (AffectedPointCode) parameters.get(Parameter.Affected_Point_Code);
	}

	public InfoString getInfoString() {
		return (InfoString) parameters.get(Parameter.INFO_String);
	}

	public NetworkAppearance getNetworkAppearance() {
		return (NetworkAppearance) parameters.get(Parameter.Network_Appearance);
	}

	public RoutingContext getRoutingContexts() {
		return (RoutingContext) parameters.get(Parameter.Routing_Context);
	}

	public void setAffectedPointCodes(AffectedPointCode afpc) {
		parameters.put(Parameter.Affected_Point_Code, afpc);
	}

	public void setInfoString(InfoString str) {
		if (str != null) {
			parameters.put(Parameter.INFO_String, str);
		}
	}

	public void setNetworkAppearance(NetworkAppearance p) {
		if (p != null) {
			parameters.put(Parameter.Network_Appearance, p);
		}
	}

	public void setRoutingContexts(RoutingContext routingCntx) {
		if (routingCntx != null) {
			parameters.put(Parameter.Routing_Context, routingCntx);
		}
	}

	public ConcernedDPC getConcernedDPC() {
		return (ConcernedDPC) parameters.get(Parameter.Concerned_Destination);
	}

	public CongestedIndication getCongestedIndication() {
		return (CongestedIndication) parameters.get(Parameter.Congestion_Indications);
	}

	public void setConcernedDPC(ConcernedDPC dpc) {
		if (dpc != null) {
			parameters.put(Parameter.Concerned_Destination, dpc);
		}
	}

	public void setCongestedIndication(CongestedIndication congInd) {
		if (congInd != null) {
			parameters.put(Parameter.Congestion_Indications, congInd);
		}
	}

	@Override
	protected void encodeParams(ByteBuffer buffer) {
		if (parameters.containsKey(Parameter.Network_Appearance)) {
			((ParameterImpl) parameters.get(Parameter.Network_Appearance)).write(buffer);
		}

		if (parameters.containsKey(Parameter.Routing_Context)) {
			((ParameterImpl) parameters.get(Parameter.Routing_Context)).write(buffer);
		}
		if (parameters.containsKey(Parameter.Affected_Point_Code)) {
			((ParameterImpl) parameters.get(Parameter.Affected_Point_Code)).write(buffer);
		}

		if (parameters.containsKey(Parameter.Concerned_Destination)) {
			((ParameterImpl) parameters.get(Parameter.Concerned_Destination)).write(buffer);
		}

		if (parameters.containsKey(Parameter.Congestion_Indications)) {
			((ParameterImpl) parameters.get(Parameter.Congestion_Indications)).write(buffer);
		}

		if (parameters.containsKey(Parameter.INFO_String)) {
			((ParameterImpl) parameters.get(Parameter.INFO_String)).write(buffer);
		}

	}

}
